import numpy as np
import pandas as pd


class EqualWeight():
    """
    Create a equal weight portfolio.

    Parameters
    ----------
    AssetPrices : :py:class:`pandas.DataFrame` with the asset prices

    Returns
    -------
    EqualWeight : :py:class:`pandas.DataFrame` with the equal weight portfolio
    """

    def __init__(self, AssetPrices: pd.DataFrame):

        # Check if the asset prices are valid
        if AssetPrices is not None:
            if not isinstance(AssetPrices, pd.DataFrame):
                raise ValueError(
                    "Asset Prices must be a Pandas DataFrame!")
            if not isinstance(AssetPrices.index, pd.DatetimeIndex):
                raise ValueError(
                    "Asset Prices must be a Pandas DataFrame index by datetime!")

        # Portfolio "construction" LMAO :D
        asset_names = AssetPrices.columns
        value = np.repeat([1 / len(asset_names)],
                          len(asset_names), axis=0)
        Portifolio = pd.Series(value, index=asset_names)
        Portifolio = Portifolio.to_frame().T.dropna(axis="columns")
        self.Portifolio = Portifolio

    def optimize(self, Decimals: int = 4):
        """
        Calculate asset weights using equal weight algorithm.

        Parameters
        ----------
        Decimals : :py:class:`int`, optional
            The number of decimals to round the weights. The default is 4.

        Returns
        -------
        Portfolio : :py:class:`pandas.DataFrame`
        """

        # "Optimize" the portfolio weights

        Portifolio = self.Portifolio
        Portifolio = Portifolio.round(Decimals)
        Portifolio /= Portifolio.sum()
        Portifolio[Portifolio == 0] = 'nan'
        Portifolio = Portifolio.dropna(axis="columns")
        self.Portifolio = Portifolio

        return Portifolio
